<?php

class CarApiClient {

    protected static $_soap_client = null;

    /**
     * Get the soap client.
     *
     * @param  boolean $renew Get a fresh soap client
     * @return SoapClient
     */
    protected static function getSoapClient($renew = false) {

        if (true === $renew || null === self :: $_soap_client) {

            self :: $_soap_client = new SoapClient(
                CARAPI_WSDL,
                array(
                    'features'   => SOAP_SINGLE_ELEMENT_ARRAYS,
                    'trace'      => true,
                    'cache_wsdl' => WSDL_CACHE_NONE,
                    'location'   => CARAPI_LOCATION
                )
            );

            self :: $_soap_client->__setSoapHeaders(
                array(
                    new SoapHeader( 'http://www.carcopy.com/WS/WXApi/1.0/', 'apikey', CARAPI_KEY)));
        }

        return self :: $_soap_client;
    }

    /**
     * Cache a certain type of result by its query.
     *
     * @param  string $type  Type of the data to be cached: vehicle/fields/languages/valuelabels/labels/values
     * @param  mixed  $query Query of the cached data.
     * @param  mixed  $result Data to be cached.
     * @return boolean
     */
    protected static function cacheResult($type = null, $query = array(), $result = null) {

        if (0 == CACHE_TIME_MAIN) {
            return false;
        }

        if (empty($type) || !is_string($type)) {
            throw new Exception('Invalid type.');
        }

        if (empty($query)) {
            throw new Exception('Invalid query.');
        }

        $cache_key = sha1(serialize($query));
        $cache_folder = CACHE_DIR . $type;
        $cache_items = serialize($result);
        $cache_file = $cache_folder . DIRECTORY_SEPARATOR . $cache_key;

        if (false == file_exists($cache_folder)) {
            mkdir($cache_folder, 0777, true);
        }

        file_put_contents($cache_file, $cache_items);

        return true;
    }

    /**
     * Get a cached result by it's query.
     *
     * @param  string $type  Type of the cached data: vehicle/fields/languages/valuelabels/labels/values
     * @param  array  $query Query of the cached data.
     * @return mixed         Cached data.
     */
    protected static function getCachedResult($type = null, $query = array()) {

        if (0 == CACHE_TIME_MAIN) {
            return false;
        }

        if (empty($type)) {
            throw new Exception('Invalid type.');
        }

        $cache_key = sha1(serialize($query));
        $cache_folder = CACHE_DIR . $type;
        $cache_file = $cache_folder . DIRECTORY_SEPARATOR . $cache_key;

        if (false == file_exists($cache_file)) {
            return false;
        }

        /*
         * Cached items timed out.
         */
        if (filemtime($cache_file) + CACHE_TIME_MAIN < time()) {
            return false;
        }

        return unserialize(file_get_contents($cache_file));
    }

    /**
     * Filter vehicle list.
     *
     * @param  stdClass $filter    [description]
     * @param  array    $fields    [description]
     * @param  integer  $start     [description]
     * @param  integer  $limit     [description]
     * @param  string   $order     [description]
     * @param  string   $orderType [description]
     * @return array
     */
    public static function filterVehicle($filter = null, $fields = array(), $start = 0, $limit = 30, $order = '',  $orderType = 'ASC') {

        $result = self :: getCachedResult('vehicle', array($filter, $fields, $start, $limit, $order, $orderType));

        if (false === $result) {

            $result = self :: getSoapClient()->doVehicleFilterView($filter, $fields, $start, $limit, $order, $orderType);
            self :: cacheResult('vehicle', array($filter, $fields, $start, $limit, $order, $orderType));
        }

        return $result;
    }


    public static function groupVehicleField($fieldName = null, $filter = null) {

        $result = self :: getCachedResult('fields', array($fieldName, $filter));

        if (false === $result) {

            $result = self :: getSoapClient()->doVehicleFieldGroupView($fieldName, $filter);
            self :: cacheResult('fields', array($fieldName, $filter), $result);
        }

        return $result;
    }


    public static function getAvailableLanguages($language = 'de') {

        $result = self :: getCachedResult('languages', array($language));

        if (false === $result) {

            $result = self :: getSoapClient()->getAvailableLanguages($language);
            self :: cacheResult('languages', array($language), $result);
        }

        return $result;
    }


    public static function getAvailableValueLabels($language = 'de') {

        $result = self :: getCachedResult('valuelabels', array($language));

        if (false === $result) {

            $result = self :: getSoapClient()->getAvailableValueLabels($language);
            self :: cacheResult('valuelabels', array($language), $result);
        }

        return $result;
    }


    public static function getLabelTranslations($language = 'de') {

        $result = self :: getCachedResult('labels', array($language));

        if (false === $result) {

            $result = self :: getSoapClient()->getLabelTranslations($language);
            self :: cacheResult('labels', array($language), $result);
        }

        return $result;
    }


    public static function getValueTranslations($label = null, $language = 'de') {

        $result = self :: getCachedResult('values', array($label, $language));

        if (false === $result) {

            $result = self :: getSoapClient()->getValueTranslations($label, $language);
            self :: cacheResult('values', array($label, $language), $result);
        }

        return $result;
    }


    public static function translateValue($value = null, $label = null) {

        if (null === $value) {
            throw new Exception('No value given.');
        }

        if (null === $label) {
            throw new Exception('No label given.');
        }

        if (substr($value, 4, 1) == 0) {
            $value = substr($value, 0, 3);
        }

        $carapi_tranlations = CarApiClient :: getValueTranslations($label);

        foreach ($carapi_tranlations->Translations as $translation) {

            if ($value == $translation->Key) {
                return $translation->Translation;
                break;
            }
        }

        return $value;
    }


    public static function translateValues($values = null, $label = null) {

        if (null === $values) {
            throw new Exception('No values given.');
        }

        if (null === $label) {
            throw new Exception('No label given.');
        }

        $carapi_tranlations = CarApiClient :: getValueTranslations($label);

        foreach ($values as &$value_item) {

            foreach ($carapi_tranlations->Translations as $translation) {

                if ($value_item->Key == $translation->Key) {
                    $value_item->Value = $translation->Translation;
                    break;
                }
            }
        }

        return $values;
    }

}

?>